""" Down Command """

from cleo import Command


class DownCommand(Command):
    """
    Puts the sever in a maintenance state

    down
    """

    def handle(self):
        with open('bootstrap/down', 'w+'):
            pass



import json
from masonite.request import Request


class JsonResponseMiddleware:

    def __init__(self, request: Request):
        self.request = request

    def after(self):
        if not self.request.header('Content-Type'):
            if isinstance(self.request.app().make('Response'), dict) or isinstance(self.request.app().make('Response'), list):
                self.request.header(
                    'Content-Type', 'application/json; charset=utf-8', http_prefix=None)
                self.request.app().bind(
                    'Response',
                    str(json.dumps(self.request.app().make('Response')))
                )
            else:
                self.request.header(
                    'Content-Type', 'text/html; charset=utf-8', http_prefix=None)



""" A RouteProvider Service Provider """

from masonite.provider import ServiceProvider
from masonite.request import Request
from masonite.routes import Route
from masonite.helpers.routes import create_matchurl


class RouteProvider(ServiceProvider):

    def register(self):
        pass

    def boot(self, router: Route, request: Request):
        # All routes joined
        for route in self.app.make('WebRoutes'):

            """Make a better match for trailing slashes
            Sometimes a user will end with a trailing slash. Because the user might
            create routes like `/url/route` and `/url/route/` and how the regex
            is compiled down, we may need to adjust for urls that end or dont
            end with a trailing slash.
            """

            matchurl = create_matchurl(router, route)

            """Houston, we've got a match
                Check to see if a route matches the corresponding router url. If a match
                is found, execute that route and break out of the loop. We only need
                one match. Routes are executed on a first come, first serve basis
            """

            if matchurl.match(router.url) and request.get_request_method() in route.method_type:
                route.load_request(request)
                if request.has_subdomain():
                    # Check if the subdomain matches the routes domain
                    if not route.has_required_domain():
                        self.app.bind('Response', 'Route not found. Error 404')
                        continue

                """Get URL Parameters
                    This will create a dictionary of parameters given. This is sort of a short
                    but complex way to retrieve the url parameters.
                    This is the code used to convert /url/@firstname/@lastname
                    to {'firstmane': 'joseph', 'lastname': 'mancuso'}.
                """

                try:
                    parameter_dict = {}
                    for index, value in enumerate(matchurl.match(router.url).groups()):
                        parameter_dict[router.generated_url_list()[index]] = value
                    request.set_params(parameter_dict)
                except AttributeError:
                    pass

                """Excute HTTP before middleware
                    Only those middleware that have a "before" method are ran.
                """

                for http_middleware in self.app.make('HttpMiddleware'):
                    located_middleware = self.app.resolve(
                        http_middleware
                    )
                    if hasattr(located_middleware, 'before'):
                        located_middleware.before()

                """Execute Before Middleware
                    This is middleware that contains a before method.
                """

                route.run_middleware('before')

                # Show a helper in the terminal of which route has been visited
                if self.app.make('Application').DEBUG:
                    print(request.get_request_method() + ' Route: ' + router.url)

                if request.get_status_code() == '404 Not Found':
                    request.status(200)

                    # Get the response from the route. This data is typically the
                    # output of the controller method
                    self.app.bind(
                        'Response',
                        route.get_response()
                    )

                """Execute After Route Middleware
                    This is middleware that contains an after method.
                """

                route.run_middleware('after')

                """Excute HTTP after middleware
                    Only those middleware that have an "after" method are ran.
                """

                for http_middleware in self.app.make('HttpMiddleware'):
                    located_middleware = self.app.resolve(
                        http_middleware
                    )
                    if hasattr(located_middleware, 'after'):
                        located_middleware.after()

                # Breaks the loop because the incoming route is found and executed.
                # There is no need to continue searching the route list.
                break
            else:
                self.app.bind('Response', 'Route not found. Error 404')



from .BaseDriver import BaseDriver
from .BaseMailDriver import BaseMailDriver
from .BaseUploadDriver import BaseUploadDriver
from .BroadcastAblyDriver import BroadcastAblyDriver
from .BroadcastPusherDriver import BroadcastPusherDriver
from .CacheDiskDriver import CacheDiskDriver
from .MailMailgunDriver import MailMailgunDriver
from .MailSmtpDriver import MailSmtpDriver
from .QueueAsyncDriver import QueueAsyncDriver
from .SessionCookieDriver import SessionCookieDriver
from .SessionMemoryDriver import SessionMemoryDriver
from .UploadDiskDriver import UploadDiskDriver
from .UploadS3Driver import UploadS3Driver



""" Base Queue Module """


class Queueable:
    """Makes classes Queueable.
    """

    def handle(self):
        """Put the queue logic in this handle method
        """

        pass

    def dispatch(self):
        """Responsible for dispatching the job to the Queue service

        Returns:
            self.handle
        """

        return self.handle



''' CSRF Middleware '''
from masonite.exceptions import InvalidCSRFToken
from masonite.request import Request
from masonite.auth import Csrf
from masonite.view import View
from jinja2 import Markup


class CsrfMiddleware:
    ''' Verify CSRF Token Middleware '''

    exempt = ['/']

    def __init__(self, request: Request, csrf: Csrf, view: View):
        self.request = request
        self.csrf = csrf
        self.view = view

    def before(self):
        token = self.__verify_csrf_token()

        self.view.share({
            'csrf_field': Markup("<input type='hidden' name='__token' value='{0}' />".format(token)),
            'csrf_token': token
        })

    def after(self):
        pass

    def __in_exempt(self):
        """
        Determine if the request has a URI that should pass
        through CSRF verification.
        """

        if self.request.path in self.exempt:
            return True
        else:
            return False

    def __verify_csrf_token(self):
        """
        Verify si csrf token in post is valid.
        """

        if self.request.is_post() and not self.__in_exempt():
            token = self.request.input('__token')
            if not self.csrf.verify_csrf_token(token):
                raise InvalidCSRFToken("Invalid CSRF token.")
        else:
            token = self.csrf.generate_csrf_token()

        return token



"""Module for controlling exceptions handling when an error occurs doing executing
code in a Masonite application. These errors could are thrown during runtime.
"""

import os
import platform
import sys
import traceback

package_directory = os.path.dirname(os.path.realpath(__file__))


class ExceptionHandler:
    """Class for handling exceptions thrown during runtime.
    """

    def __init__(self, app):
        """ExceptionHandler constructor. Also responsible for loading static files into the container.

        Arguments:
            app {masonite.app.App} -- Container object
        """

        self._app = app

        self._register_static_files()

    def _register_static_files(self):
        """Register static files into the container.
        """

        self._app.make('Storage').STATICFILES.update(
            {
                os.path.join(package_directory, 'snippets/exceptions'):
                '_exceptions/'
            }
        )

    def load_exception(self, exception):
        """Loads the exception thrown into this handler.

        Arguments:
            exception {Exception} -- This is the exception object thrown at runtime.
        """

        self._exception = exception

        self._render()

    def _render(self):
        """Render an exception view if the DEBUG configuration is True. Else this should not return anything.

        Returns:
            None
        """

        self._app.bind('StatusCode', '500 Internal Server Error')

        # Run Any Framework Exception Hooks
        self._app.make('HookHandler').fire('*ExceptionHook')

        # Check if DEBUG is False
        if not self._app.make('Application').DEBUG:
            return

        # return a view
        rendered_view = self._app.make('View')('/masonite/snippets/exception',
                                               {
                                                   'exception': self._exception,
                                                   'traceback': traceback,
                                                   'tb': sys.exc_info(),
                                                   'app': self._app,
                                                   'enumerate': enumerate,
                                                   'open': open,
                                                   'platform': platform
                                               }
                                               ).rendered_template
        self._app.bind('Response', rendered_view)



""" A SassProvider Service Provider """

from masonite.provider import ServiceProvider
from masonite.storage import Storage


class SassProvider(ServiceProvider):

    wsgi = False

    def register(self):

        """Compile Sass
            Compile Sass if the libsass module is installed. Once installed, all
            Sass files are compiled when the server is ran. This will only run
            once when the server is first started.
        """

        Storage().compile_sass()

    def boot(self):
        pass



""" CSRF Protection Module """
import binascii
import os


class Csrf:
    """CSRF Protection Class
    """

    def __init__(self, request):
        """CSRF constructor

        Arguments:
            request {masonite.request.Request} -- Request object
        """

        self.request = request

    def generate_csrf_token(self):
        """Generate CRSRF token

        Returns:
            string -- Returns token generated
        """

        token = bytes(binascii.b2a_hex(os.urandom(15))).decode('utf-8')
        self.request.cookie('csrf_token', token, encrypt=False)
        return token

    def verify_csrf_token(self, token):
        """Verify if csrf token is valid from the cookie set.

        Arguments:
            token {string} -- The token that was generated.

        Returns:
            bool
        """

        if self.request.get_cookie('csrf_token', decrypt=False) == token:
            return True
        else:
            return False



""" A Cache Service Provider """

from config import cache
from masonite.drivers import CacheDiskDriver
from masonite.managers import CacheManager
from masonite.provider import ServiceProvider


class CacheProvider(ServiceProvider):

    wsgi = False

    def register(self):
        self.app.bind('CacheConfig', cache)
        self.app.bind('CacheDiskDriver', CacheDiskDriver)
        self.app.bind('CacheManager', CacheManager(self.app))

    def boot(self, cache: CacheManager):
        self.app.bind('Cache', cache.driver(self.app.make('CacheConfig').DRIVER))



""" Queue Manager Module """

from masonite.contracts import QueueManagerContract
from masonite.managers import Manager


class QueueManager(Manager, QueueManagerContract):
    """Manages all queue drivers.

    Arguments:
        Manager {masonite.managers.Manager} -- The base Manager class.
    """

    config = 'QueueConfig'
    driver_prefix = 'Queue'



"""Module for the LoadEnvironment class.
"""

import os
from pathlib import Path

from dotenv import load_dotenv


class LoadEnvironment:
    """This class is used for loading the environment from .env and .env.* files.
    """

    def __init__(self, env=None, override=True, only=None):
        """LoadEnvironment constructor.

        Keyword Arguments:
            env {string} -- An additional environment file that you want to load. (default: {None})
            override {bool} -- Whether or not the environment variables found should overwrite existing ones. (default: {False})
            only {string} -- If this is set then it will only load that environment. (default: {None})
        """

        if only:
            self._load_environment(only, override=override)
            return

        env_path = str(Path('.') / '.env')
        load_dotenv(env_path, override=override)

        if os.environ.get('APP_ENV'):
            self._load_environment(os.environ.get(
                'APP_ENV'), override=override)
        if env:
            self._load_environment(env, override=override)

    def _load_environment(self, env, override=False):
        """Load the environment depending on the env file.

        Arguments:
            env {string} -- Name of the environment file to load from

        Keyword Arguments:
            override {bool} -- Whether the environment file should overwrite existing environment keys. (default: {False})
        """

        env_path = str(Path('.') / '.env.{}'.format(env))
        load_dotenv(dotenv_path=env_path, override=override)
